@@ -1,4 +1,4 @@
-## Copyright (C) 2009-2019 Philip Nienhuis
+## Copyright (C) 2009-2021 Philip Nienhuis
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -71,7 +71,7 @@
 ## Only proper operation of main interface groups (COM, Java) is shown.
 ## If @var{path_to_jars} and/or @var{path_to_ooo} was supplied,
 ## chk_spreadsheet_support indicates whether it could find the required
-## Java class libs for all interfaces
+## Java class libs for all interfaces.
 ##
 ## @item 2
 ## Like 1, proper working of individual implemented Java-based interfaces is
@@ -88,7 +88,7 @@
 ## @item -1 (or any negative number)
 ## Remove all directories and Java class libs that chk_spreadsheet_support
 ## added to the javaclasspath. If @var{debug_level} < 1 report number of 
-## removed javclasspath entries; if @var{debug_level} < 2 report each
+## removed javaclasspath entries; if @var{debug_level} < 2 report each
 ## individual removed entry.
 ## @noindent
 ## @end table
@@ -273,7 +273,7 @@ function  [ retval, s_interfaces, jars_loaded ]  = chk_spreadsheet_support (path
   if (! tmp1)
     ## We can return as for below code Java is required.
     if (dbug > 1)
-      printf ("No Java support found.\n");
+      printf ("No Java support found, maybe Java's bitness doesn't match Octave's.\n");
       if (! retval)
         printf ("Only ODS 1.2 (.ods) & OOXML (.xlsx) r/w support & .gnumeric read support present\n");
       endif
@@ -370,7 +370,7 @@ function  [ retval, s_interfaces, jars_loaded ]  = chk_spreadsheet_support (path
     endif
     ## First combine all entries
     targt = numel (missing);
-    ## For each interface, search tru list of missing entries
+    ## For each interface, search thru list of missing entries
     for ii=1:6   ## Adapt number in case of future new interfaces
       tmpm = eval ([ "missing" char(ii + "0") ]);
       tmpmcnt = numel (tmpm);
@@ -476,7 +476,7 @@ function  [ retval, s_interfaces, jars_loaded ]  = chk_spreadsheet_support (path
     endif
     if (isempty (uredir))
       if (dbug > 1)
-        warning ("Wrong path to OOo/LO, or incomplete OOo/LO installation\n");
+        warning ("chk_spreadsheet_support:\nWrong path to OOo/LO, or incomplete OOo/LO installation\n");
       endif
       return
     endif
