@@ -1,4 +1,4 @@
-## Copyright (C) 2009-2018 P.R. Nienhuis
+## Copyright (C) 2009-2021 P.R. Nienhuis
 ## parts Copyright (C) 2007 Michael Goffioul
 ##
 ## This program is free software; you can redistribute it and/or modify it under
@@ -48,7 +48,7 @@ function [rawarr, xls, rstatus ] = __COM_spsh2oct__ (xls, wsh, crange, spsh_opts
     error ("__COM_spsh2oct__ needs a minimum of 2 arguments."); 
   endif
   if (size (wsh, 2) > 31) 
-    warning ("Worksheet name too long - truncated\n") 
+    warning ("xls2oct: worksheet name too long - truncated\n") 
     wsh = wsh(1:31);
   endif
   app = xls.app;
@@ -57,7 +57,7 @@ function [rawarr, xls, rstatus ] = __COM_spsh2oct__ (xls, wsh, crange, spsh_opts
   try
     wb_cnt = wb.Worksheets.count;
   catch
-    error ("ActiveX invocation in file ptr struct seems non-functional");
+    error ("xls2oct: ActiveX invocation in file ptr struct seems non-functional");
   end_try_catch
 
   ## Check & get handle to requested worksheet. Take ""Sheets as the total
@@ -66,7 +66,7 @@ function [rawarr, xls, rstatus ] = __COM_spsh2oct__ (xls, wsh, crange, spsh_opts
   old_sh = 0;  
   if (isnumeric (wsh))
     if (wsh < 1 || wsh > wb_cnt)
-      errstr = sprintf ("Sheet number: %d out of range 1-%d", wsh, wb_cnt);
+      errstr = sprintf ("xls2oct: sheet number: %d out of range 1-%d", wsh, wb_cnt);
       error (errstr)
       rstatus = 1;
       return
@@ -83,7 +83,7 @@ function [rawarr, xls, rstatus ] = __COM_spsh2oct__ (xls, wsh, crange, spsh_opts
       endif
     endfor
     if (! old_sh)
-      errstr = sprintf ("Worksheet name \"%s\" not present", wsh);
+      errstr = sprintf ("xls2oct: worksheet name \"%s\" not present", wsh);
       error (errstr)
     else
       wsh = old_sh;
@@ -130,12 +130,37 @@ function [rawarr, xls, rstatus ] = __COM_spsh2oct__ (xls, wsh, crange, spsh_opts
       rawarr = {rawarr};
     endif
 
+    if (spsh_opts.convert_utf) ...
+        && (compare_versions (ver ("windows").Version, "1.5.0", "<="))
+      ## The COM interface in of-windows until version 1.5.0 returns strings
+      ## that are encoded in the system locale.  All character values not
+      ## contained in the locale encoding are mapped to codepoint 32 (space
+      ## character).  Try to recover the remainder.
+      if (isempty (which ("native2unicode")))
+        ## Not available before Octave 4.4
+        ## Use fallback function. (Assumes the locale is ISO 8859-1.)
+        conv_fcn = @unicode2utf8;
+      else
+        if (isempty (which ("__locale_charset__")))
+          ## Not available before Octave 6
+          ## Assume ISO 8859-1 (Latin-1)
+          enc = "iso8859-1";
+        else
+          enc = __locale_charset__ ();
+        endif
+        conv_fcn = @(str) native2unicode (uint8 (str), enc);
+      endif
+
+      idx = cellfun (@ischar, rawarr);
+      rawarr(idx) = cellfun (conv_fcn, rawarr(idx), "uniformoutput", false);
+    endif
+    
     ## If we get here, all seems to have gone OK
     rstatus = 1;
     ## Keep track of data rectangle limits
     xls.limits = [lcol, rcol; trow, brow];
   else
-    error ("No data read from spreadsheet file");
+    warning ("xls2oct: no data read from spreadsheet file");
     rstatus = 0;
   endif
   
